//#include maps\mp\_utility;

Run()
{
	game_state = gm\Variables::GetGMVar("game_state", "playing");
	gm\Variables::SetGMVar("game_state", "readyup");
	
	ServerReadyUpLoop();	
	
	gm\Variables::SetGMVar("gm_readyup_active", false);
	
	gm\Variables::SetGMVar("game_state", game_state);
	
	ShowCountdown();
	level notify("restarting");
	game["state"] = "playing";
    map_restart(true);
}

PlayerReadyUpLoop()
{
	self endon("disconnect");
	
	readyup_status_hud = gm\HUD::CreateClientHUDElement(self, 320, 100, "center", "middle", "default", 1.4, true, "Not Ready", (1, 0, 0));	
	self gm\HUD::AddPlayerHUDElement("player_readyup_status_hud", readyup_status_hud);
	
	if (gm\Variables::GetGMVar("overtime", false))
		self thread SideSelection();		
	
	if (gm\Variables::GetGMVar("gm_readyup_show_enemy_guids"))
		self thread GuidDisplayLoop();
	
	while(level.ReadyUpActive)
	{
		if (self UseButtonPressed() == true)
		{
			self.Ready = !self.Ready;
			
			if (self.Ready)
			{
				readyup_status_hud SetText("Ready");
				readyup_status_hud.Color = (0, 1, 0);
			}
			else
			{
				readyup_status_hud SetText("Not Ready");
				readyup_status_hud.Color = (1, 0, 0);
			}			
			
			// wait for the player to release the use button
			while (self UseButtonPressed() == true && level.ReadyUpActive)
				wait .1;
		}
		
		wait .01;
	}
	
	self gm\HUD::RemovePlayerHUDElement("player_readyup_status_hud");
	if (isDefined(readyup_status_hud))
		readyup_status_hud destroy();
}

ServerReadyUpLoop()
{
	// a lot of this was taken from PAM 4.02
	level.ReadyUpActive = true;

	while (level.ReadyUpActive)
	{				
		not_ready_count = 0;

		players = getentarray("player", "classname");
		
		//at least one player thats non spectator
		at_least_one_player = false;

		// See if there are any players on the server
		if (players.size < 1)
		{
			wait 1;
			continue;
		}

		// Go through all the players
		for(i = 0; i < players.size; i++)
		{
			player = players[i];

			//Check to see if player is scanning
			if (isDefined(player.pers["team"]) && player.pers["team"] != "spectator")
			{
				if (!isDefined(player.IsScanning) && !player.IsScanning)
				{
					player.IsScanning = true;
					player.ready = false;
					player thread PlayerReadyUpLoop();
				}

				if (!player.ready)
				{
					not_ready_count++;
				}
				
				at_least_one_player = true;
			}
		}

		if (not_ready_count == 0 && at_least_one_player)
			level.ReadyUpActive = false;
			
		wait .1;
	}
}

ShowCountdown()
{	
	wait 0.2;
	readyup_countdown_hud = gm\HUD::CreateHUDElement(320, 100, "center", "middle", "default", 1.4, true, "All players are ready", (1, 1, 0));
	wait 1.0;
	
	for(i = 3; i > 0; i--)
	{
		readyup_countdown_hud SetText("Starting in: " + i);
		wait 1;
	}
	
	if (isDefined(readyup_countdown_hud))
		readyup_countdown_hud destroy();
}

//-----------------------------------------------------------------------------------------------
//
//-----------------------------------------------------------------------------------------------
CreateGuidDisplay()
{	
	readyup_guid_hud = gm\HUD::CreateClientHUDElement(self, 1, 200, "left", "middle", "default", 1.0, true, "Guids:", (1, 1, 0));
	self gm\HUD::AddPlayerHUDElement("player_guid_display", readyup_guid_hud);
}

GuidDisplayMonitor()
{	
	self endon("disconnect");
	while(level.ReadyUpActive)
	{
		self waittill("spawned_player");
		
		players = getentarray("player", "classname");
		for(i = 0; i < players.size; i++)
		{
			players[i] UpdateGuidDisplay();		
		}		
	}
}

GuidDisplayLoop()
{
	self endon("disconnect");
	
	self thread GuidDisplayMonitor();
	display_active = false;
	
	while(level.ReadyupActive)
	{	
		if (self ActionSlotTwoButtonPressed() == true)
		{
			display_active = !display_active;
			
			if(display_active)
			{
				self CreateGuidDisplay();	
				self UpdateGuidDisplay();
			}
			else
			{				
				self DestroyGuidDisplay();
			}
			// wait for the player to release the use button
			while (self ActionSlotTwoButtonPressed() == true && level.ReadyUpActive)
				wait .1;
		}
		wait .05;
	}
	
	self DestroyGuidDisplay();
}

UpdateGuidDisplay()
{
	guid_hud = self gm\HUD::GetPlayerHUDElement("player_guid_display");
	if (isDefined(guid_hud))
	{
		team = self.pers["team"];
		display_string = "Guids:\n";
		
		players = getentarray("player", "classname");
		for(i = 0; i < players.size; i++)
		{
			if (players[i].pers["team"] != team && players[i].pers["team"] != "spectator")
			{
				guid = players[i] GetGUID();
				display_string += (players[i].name + " - " + guid + "\n");
			}
		}		
		guid_hud SetText(display_string);
	}
}

DestroyGuidDisplay()
{
	self gm\HUD::RemovePlayerHUDElement("player_guid_display");
}
//-----------------------------------------------------------------------------------------------
//
//-----------------------------------------------------------------------------------------------
SideSelection()
{	
	team = self.pers["team"];
	
	//switch the teams if sides are already switched
	if (game["switchedsides"])
		team = maps\mp\_utility::getOtherTeam(team);
		
	side_selection_hud = gm\HUD::CreateClientHUDElement(self, 320, 60, "center", "middle", "default", 1.4, true, "Starting Side: " + game[team], (0, 0, 0));
	self gm\HUD::AddPlayerHUDElement("player_side_selection_hud", side_selection_hud);
			
	while(level.ReadyUpActive)
	{
		if (self ActionSlotThreeButtonPressed() == true)
		{			
			team = maps\mp\_utility::getOtherTeam(team);
						
			game["switchedsides"] = !game["switchedsides"];
			gm\_Util::NotifyUser("Sides switched. Readyup Reset");
			side_selection_hud SetText("Starting Side: " + game[team]);
			ReadyUpResetAll();
			
			// wait for the player to release the use button
			while (self ActionSlotThreeButtonPressed() == true && level.ReadyUpActive)
				wait .1;
		}		
		
		wait .01;
	}
	
	self gm\HUD::RemovePlayerHUDElement("player_side_selection_hud");
	if (side_selection_hud)
		side_selection_hud destroy();	
}

ReadyUpResetAll()
{
	players = getentarray("player", "classname");

	// Go through all the players
	for(i = 0; i < players.size; i++)
	{
		player = players[i];

		player ReadyUpReset();
	}
}

ReadyUpReset()
{
	team = self.pers["team"];
			
	self.ready = false;
		
	readyup_status_hud = self gm\HUD::GetPlayerHUDElement("player_readyup_status_hud");
	
	if (isDefined(readyup_status_hud))
	{
		if (team == "spectator")
		{
			self gm\HUD::RemovePlayerHUDElement("player_readyup_status_hud");
			readyup_status_hud destroy();
		}
		else
		{
			readyup_status_hud SetText("Not Ready");
			readyup_status_hud.Color = (1, 0, 0);
		}
	}
		
	if (gm\Variables::GetGMVar("overtime", false))
	{
		side_selection_hud = self gm\HUD::GetPlayerHUDElement("player_side_selection_hud");
	
		if (isDefined(side_selection_hud))
		{
			if (team == "spectator")
			{
				self gm\HUD::RemovePlayerHUDElement("player_side_selection_hud");
				side_selection_hud destroy();
			}
			else
			{
				if (game["switchedsides"])
					team = maps\mp\_utility::getOtherTeam(team);
					
				side_selection_hud SetText("Starting Side: " + game[team]);
			}
		}
	}
}