#include gm\_Util;
#include gm\HUD;
#include gm\Variables;

TimeoutInit()
{
	timeouts = GetGMVar("gm_timeout_count", 0);
	
	SetGMVar("allies_timeouts_left", timeouts);
	SetGMVar("axis_timeouts_left", timeouts);
}

TimeoutMonitor()
{	
	while(true)
	{
		if (self ActionSlotTwoButtonPressed() && self CanCallTimeout())
			CallTimeout();
			
		wait 0.01;
	}
}

CanCallTimeout()
{
	team = self.pers["team"];
	if (team == "spectator")
		return false;
		
	if (GetGMVar("game_state", "readyup") == "readyup")
		return false;
		
	//we do not want to allow timeouts during major events in gametypes
	//like someone running with the flag, a bomb is planted or an hq is capped
	switch(level.gametype)
	{
		case "sab":
			if (isDefined(level.bombPlanted) && level.bombPlanted)
				return false;
			break;
		case "ctf":
			if (!level.teamFlags[ "allies" ] maps\mp\gametypes\ctf::isHome() || !level.teamFlags[ "axis" ] maps\mp\gametypes\ctf::isHome())
				return false;
			break;
		case "dem":
			if (isDefined(level.bombAPlanted) && level.bombAPlanted)
				return false;
			if (isDefined(level.bombBPlanted) && level.bombBPlanted)
				return false;
			break;
		case "koth":
			if (isDefined(level.radioCaptured) && level.radioCaptured)
				return false;
			break;
	}	
	
	timeouts_left = GetGMVar(team + "_timeouts_left", 0);
	
	return timeouts_left > 0;
}

CallTimeout()
{	
	team = self.pers["team"];
	if (team == "spectator")
		return false;
		
	timeouts_left = GetGMVar(team + "_timeouts_left", 0);
	if (timeouts_left == 0)
	{
		LogError("timeout called with no timeouts left");
		return;
	}
	
	timeouts_left = timeouts_left - 1;
	SetGMVar(team + "_timeouts_left", timeouts_left);
	LogInfo("Timeout called by " + self.name + " , " + timeouts_left + " timeouts left");
	
	SetGMVar("timeout_activated", true);
	
	//check gametype.. call immediately for non round gametypes
	if (level.gametype != "sd")
	{
		timeout_duration = GetGMVar("gm_timeout_duration", 60);
		self thread ActivateTimeout(timeout_duration);
	}
}

ActivateTimeout(Duration)
{
	//level notify("gm_timeout_activated", Duration);
	game_state = GetGMVar("game_state", "playing");
	SetGMVar("game_state", "timeout");
	
	players = getentarray("player", "classname");
	for(i = 0; i < players.size; i++)
	{
		players[i] freezecontrols(true);
		players[i] EnableInvulnerability();
		anchor = spawn("script_origin", players[i].origin);
		players[i] linkto(anchor);

		//SetGMEntVar(players[i], "timeout_last_pos", players[i].origin);		
	}
	
	level.timeLimitOverride = true;
	maps\mp\gametypes\_globallogic_utils::pauseTimer();
	setGameEndTime(0);
	
	InitTimeoutHUD();
	if (Duration > 0)
	{
		count = Duration;
					
		while (count > 0)
		{
			if (self usebuttonpressed())
				break;
			
			wait 0.1;
			count -= 0.1;
			
			UpdateTimeoutHUD(int(count));
		}		
	}
	else
	{
		//wait for user to press time in
		while (true)
		{
			if (self usebuttonpressed())
				break;
							
			wait 0.1;
		}
	}
	
	TimeoutResumeCountdown();
	
	SetGMVar("timeout_activated", false);
	DestroyTimeoutHUD();
	
	//regrab the player list in case we get a new player
	//(typically timeouts are used when a player drops)
	players = getentarray("player", "classname");
	for(i = 0; i < players.size; i++)
	{
		players[i] unlink();
		players[i] DisableInvulnerability();
		players[i] freezecontrols(false);		
	}
	
	SetGMVar("game_state", game_state);
	maps\mp\gametypes\_globallogic_utils::resumeTimer();
	level.timeLimitOverride = false;
}

InitTimeoutHUD()
{
	AddHUDElement("timeout_hud", CreateHUDElement(320, 100, "center", "middle", "default", 1.4, true, "Timeout (Press Use To Resume)", (0, 0, 0)));
}

DestroyTimeoutHUD()
{	
	RemoveHUDElement("timeout_hud");
}

UpdateTimeoutHUD(Timeleft)
{
	hud = GetHUDElement("timeout_hud");
	if (isDefined(hud))
		hud setText("Timeout (Press Use To Resume): " + TimeLeft);	
}

TimeoutResumeCountdown()
{
	hud = GetHUDElement("timeout_hud");
	if (isDefined(hud))
	{
		time = 3;
		while (time > 0)
		{
			hud setText("Resuming in: " + time);	
			wait 1.0;
			time -= 1;
		}
	}
}